# -*- coding: utf-8 -*-

import sys
import os
import time
import json
import glob
import logging
import logging.config
import pymysql
import shutil
import zipfile
from operator import itemgetter
import datetime
from artix.database.info import DBInfo

logging.config.fileConfig('/linuxcash/cash/bin/maint_log.ini')
logger = logging.getLogger('su.artix.maintservice')

def getDate(direct):
    dateformat = direct['dateformat'] if 'dateformat' in direct.keys() else '%Y-%m-%d %H:%M:%S'
    return datetime.datetime.now().strftime(dateformat)


def database(data):
    dbInfo = DBInfo()
    conn = pymysql.connect(host=dbInfo.host, port=dbInfo.port, user=dbInfo.user, password=dbInfo.password, database="documents")
    cur = conn.cursor()
    cur.execute('SET NAMES utf8')

    if 'queries' in data.keys():
        for query in data['queries']:
            q = query % vars()
            logger.info(u"Выполнение запроса '%s'" % (q))
            cur.execute(q)
            conn.commit()

    cur.close()
    conn.close()


def sort_files_by_date_created(files):
    fileData = {}
    for fname in files:
        fileData[fname] = os.stat(fname).st_mtime

    listObj = []
    for f in sorted(fileData.items(), key=itemgetter(1)):
        listObj.append(f[0])
    return listObj


def getObjectsForAction(rangetype, rangecount, objects):

    objs = sort_files_by_date_created(objects)
    rangetype = rangetype.upper()

    if rangetype == 'EXCEPTLAST':
        if len(objs) > rangecount:
            return objs[:len(objs) - rangecount]
    elif rangetype == 'EXCEPTFIRST':
        if len(objs) > rangecount:
            return objs[rangecount:]
    elif rangetype == 'FIRST':
        if len(objs) > rangecount:
            return objs[:rangecount]
        else:
            return objs
    elif rangetype == 'LAST':
        if len(objs) > rangecount:
            return objs[len(objs) - rangecount:len(objs)]
        else:
            return objs

    return []


def createDir(path):
    if os.path.exists(path):
        return True

    try:
        os.makedirs(path)
    except OSError:
        logger.info(u"Не удалось создать директорию: %s" % path)
        return False
    return True


def delete(listObjects):

    for obj in listObjects:
        logger.info(u"Удаление '%s'" % (obj))
        if os.path.isdir(obj):
            shutil.rmtree(obj)
        elif os.path.isfile(obj):
            os.remove(obj)
        elif os.path.islink(path):
            os.unlink(obj)


def rename(listObjects, direct):

    shift = direct['shift']
    cashcode = direct['cashcode']
    objectName = direct['objectName']
    currentdatetime = getDate(direct)
    for obj in listObjects:
        newname = os.path.join(os.path.dirname(obj), direct['transform'] % vars())

        if objectName == "directory" and not createDir(newname):
            continue
        elif objectName == "file" and not createDir(os.path.dirname(newname)):
            continue

        try:
            os.rename(obj, newname)
        except OSError as e:
            logger.info("%s. Error on path: %s" % (e, newname))
            continue

        logger.info("rename '%s' -> '%s'" % (obj, newname))

def move(listObjects, direct):

    shift = direct['shift']
    cashcode = direct['cashcode']
    currentdatetime = getDate(direct)
    for obj in listObjects:
        srcname = os.path.basename(obj)
        newname = os.path.join(direct['destination'], direct['transform'] % vars())

        if not createDir(os.path.dirname(newname)):
            continue
        if os.path.exists(newname):
            logger.info("File '%s' wasn't moved, destination file path '%s' exists" % (obj, newname))
            continue

        try:
            os.rename(obj, newname)
        except OSError as e:
            logger.info("%s. Error on path: %s" % (e, newname))
            continue

        logger.info("rename '%s' -> '%s'" % (obj, newname))


def copy(listObjects, destination):

    for obj in listObjects:
        newname = os.path.join(destination, os.path.basename(obj))
        if os.path.isdir(obj):
            shutil.copytree(obj,  newname)
            logger.info("copy '%s' -> '%s'" % (obj, newname))
        elif os.path.isfile(obj):
            shutil.copy(obj, newname)
            logger.info("copy '%s' -> '%s'" % (obj, newname))


def zipfolder(path, relname, archive):

    paths = os.listdir(path)
    if len(paths) > 0:
        for p in paths:
            p1 = os.path.join(path, p)
            p2 = os.path.join(relname, p)
            if os.path.isdir(p1):
                zipfolder(p1, p2, archive)
            else:
                archive.write(p1, p2)
    else:
        archive.write(path, relname)   # empty directory


def compress(listObjects, direct):

    shift = direct['shift']
    cashcode = direct['cashcode']
    currentdatetime = getDate(direct)
    zipfilename = direct['transform'] % vars() + '.zip'

    archive = zipfile.ZipFile(os.path.join(direct['destination'], zipfilename), "w", zipfile.ZIP_DEFLATED)

    for obj in listObjects:
        if os.path.isdir(obj):
            zipfolder(obj, os.path.basename(obj), archive)
        else:
            archive.write(obj, os.path.basename(obj))

        logger.info("archive '%s'" % (obj))

    archive.close()


def directory(direct):

    if not 'path' in direct.keys() or not 'action' in direct.keys():
        return

    logger.info(u"Работа с директорией: '%s', действие: '%s'" % (direct['path'], direct['action'].upper()))

    path = os.path.join(direct['path'], direct['mask']) if 'mask' in direct.keys() else direct['path']
    listObjects = glob.glob(path)

    if len(listObjects) < 1:
        return

    if direct['action'].upper() in ['COMPRESS', 'DELETE', 'COPY']:
        if 'rangetype' in direct.keys():
            listObjects = getObjectsForAction(direct['rangetype'], int(direct['rangecount']), listObjects)
        else:
            listObjects = sort_files_by_date_created(listObjects)

    action = direct['action'].upper()

    if action == 'DELETE':
        delete(listObjects)
    elif action == 'RENAME':
        direct['objectName'] = 'directory'
        rename(listObjects, direct)
    elif action == 'MOVE':
        move(listObjects, direct)
    elif action == 'COPY':
        copy(listObjects, direct['destination'])
    elif action == 'COMPRESS':
        compress(listObjects, direct)


def file(obj):

    if not 'name' in obj.keys() or not 'action' in obj.keys():
        return
    filename = obj['name'] % obj
    files = sort_files_by_date_created(glob.glob(filename + '*' + obj['filter']))

    logger.info("work with file '%s', action = '%s" % (filename, obj['action'].upper()))

    action = obj['action'].upper()
    if action == 'COPY':
        copy(files, obj['destination'])
    elif action == 'RENAME':
        obj['objectName'] = 'file'
        rename(files, obj)


def main(argv):
    os.system('clear')

    process_what = "NONE"

    if len(argv) == 1:
        process_what = str(argv[0]).lower()

    with open('/linuxcash/cash/data/cash.reg') as infile:
        registry = json.load(infile)

    shift = registry["shiftNumber"]

    if not process_what in ['shiftly', 'weekly', 'custom']:
        logger.error("Argument must be 'SHIFTLY', 'WEEKLY' or 'CUSTOM'")
        return 1

    if process_what == 'shiftly':
        shift = shift - 1

    cashcode = str(registry["cashCode"])
    shift = "%06d" % shift

    logger.info("shift=%s, cashcode=%s" % (shift, cashcode))

    with open('/linuxcash/cash/conf/maint.json') as data_file:
        data = json.load(data_file)

    if not process_what in data.keys():
        logger.error("File 'maint.json' has not key '%s'" % (process_what))
        return 1

    for obj in data[process_what]:
        if not 'objectname' in obj.keys():
            continue

        if 'destination' in obj.keys() and not createDir(obj['destination']):
            continue

        obj['shift'] = shift
        obj['cashcode'] = cashcode

        if obj['objectname'] == 'directory':
            directory(obj)
        elif obj['objectname'] == 'file':
            file(obj)
        elif obj['objectname'] == 'database':
            database(obj)

if __name__ == "__main__":
    main(sys.argv[1:])
